@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 # Modify the config.json.sample file, save it to config.json. Run server.pl on
-# server, run client.pl on localhost. Config your beowser to use socks 5 proxy
+# server, run client.pl on localhost. Config your browser to use socks 5 proxy
 # with remote DNS support and you can now break through the great firewall of
 # china.
 
@@ -8,7 +8,7 @@ use 5.006;
 use strict;
 use warnings;
 use JSON;
-use Net::Shadowsocks;
+use Net::Shadowsocks::Client;
 
 local $/;
 open(my $fh, '<', 'config.json') or die "Can't open  config.json: $!";
