@@ -4,17 +4,55 @@ package Net::Shadowsocks;
 use 5.006;
 use strict;
 use warnings;
+use AnyEvent::Log;
 use Carp;
+use Crypt::Cipher::AES;
+use Crypt::Cipher::Camellia;
 use Crypt::Cipher::RC6;
 use Crypt::Mode::CBC;
-use Crypt::Nettle::Cipher;
+use Crypt::Mode::CFB;
+use Crypt::Mode::CTR;
+use Crypt::NaCl::Sodium;
+#use Crypt::Nettle::Cipher;
 use Crypt::Random qw( makerandom_octet );
 use Digest::MD5;
-use Mcrypt qw(ARCFOUR RIJNDAEL_128 RIJNDAEL_256);
+use Mcrypt qw(ARCFOUR RIJNDAEL_128 RIJNDAEL_192 RIJNDAEL_256);
 use Mcrypt qw(:MODES);
 use Mcrypt qw(:FUNCS);
 
-our $VERSION = '0.7.1';
+our $VERSION = '0.7.2';
+
+our %_ciphers = 
+    (
+        "rc4-md5"  => [Mcrypt::ARCFOUR,Mcrypt::STREAM,16,16],
+        "rc4-sha" => [Mcrypt::ARCFOUR,Mcrypt::STREAM,16,20],
+        "aes-128-cbc"  => [Mcrypt::RIJNDAEL_128,Mcrypt::CBC,16,16],
+        "aes-128-cfb"  => ['AES','cfb',16,16],
+        "aes-128-ctr"  => ['AES','ctr',16,16],
+        "aes-128-ofb" => [Mcrypt::RIJNDAEL_128,Mcrypt::OFB,16,16],
+        "aes-192-cbc"  => [Mcrypt::RIJNDAEL_192,Mcrypt::CBC,24,16],     
+        "aes-192-cfb" => ['AES','cfb',24,16],
+        "aes-192-ctr"  => ['AES','ctr',24,16],
+        "aes-192-ofb" => [Mcrypt::RIJNDAEL_192,Mcrypt::OFB,24,16],
+        "aes-256-cbc"  => [Mcrypt::RIJNDAEL_256,Mcrypt::CBC,32,16],
+        "aes-256-cfb"  => ['AES','cfb',32,16],
+        "aes-256-ctr"  => ['AES','ctr',32,16],
+        "aes-256-ofb"  => [Mcrypt::RIJNDAEL_256,Mcrypt::OFB,32,16]
+        ,"camellia-128-cbc" => ['Camellia','cbc',16,16],
+        "camellia-128-cfb" => ['Camellia','cfb',16,16],
+         "camellia-128-ctr" => ['Camellia','ctr',16,16],
+          "camellia-128-ofb" => ['Camellia','ofb',16,16],
+          "camellia-192-cbc" => ['Camellia','cbc',24,16],
+        "camellia-192-cfb" => ['Camellia','cfb',24,16],
+        "camellia-192-ctr" => ['Camellia','ctr',24,16],
+        "camellia-192-ofb" => ['Camellia','ofb',24,16],
+        "camellia-256-cbc" => ['Camellia','cbc',32,16],
+        "camellia-256-cfb" => ['Camellia','cfb',32,16],
+        "camellia-256-ctr" => ['Camellia','ctr',32,16],
+        "camellia-256-ofb" => ['Camellia','ofb',32,16],
+        "chacha20-ietf" => [undef,undef,32,12],
+        "rc6" => [undef,undef,8,16]
+    );
 
 sub _EVP_BytesToKey($$$$$) 
 {
@@ -58,20 +96,6 @@ sub _EVP_BytesToKey($$$$$)
 
 sub _initialize_cipher($$) 
 {
-        my %ciphers = 
-    (
-        "rc4-md5"  => [Mcrypt::ARCFOUR,Mcrypt::STREAM,16,16],
-        "rc4-sha" => [Mcrypt::ARCFOUR,Mcrypt::STREAM,16,20],
-        "aes-128-cbc"  => [Mcrypt::RIJNDAEL_128,Mcrypt::CBC,16,16],
-        "aes-128-cfb"  => [Mcrypt::RIJNDAEL_128,Mcrypt::CFB,16,16],
-        #"aes-128-ctr"  => [Mcrypt::RIJNDAEL_128,Mcrypt::CTR,16,16],
-        "aes-128-ofb" => [Mcrypt::RIJNDAEL_128,Mcrypt::OFB,16,16],
-        "aes-256-cbc"  => [Mcrypt::RIJNDAEL_256,Mcrypt::CBC,32,16],
-        "aes-256-cfb"  => [Mcrypt::RIJNDAEL_256,Mcrypt::CFB,32,16],
-        #"aes-256-ctr"  => [Mcrypt::RIJNDAEL_256,Mcrypt::CTR,32,16],
-        "aes-256-ofb"  => [Mcrypt::RIJNDAEL_256,Mcrypt::OFB,32,16],
-        "rc6" => [undef,undef,8,16]
-    );
     my $_method   = shift;
     my $_password = shift;
 
@@ -80,111 +104,139 @@ sub _initialize_cipher($$)
     my $_key;
     my $_iv;
     my $_nonce;
-    if ( $_method eq 'rc6') 
-    {
-        $_key = Net::Shadowsocks::_EVP_BytesToKey( 16, 16, '', $_password, 1 );
-        $_iv = makerandom_octet( Length => $ciphers{$_method}->[3] );
-        $_encryptor = Crypt::Mode::CBC->new('RC6');
-        $_decryptor = Crypt::Mode::CBC->new('RC6');
-    }
-    elsif(defined($_method))
-    {
-        $_encryptor = Mcrypt::mcrypt_load($ciphers{$_method}->[0],'',$ciphers{$_method}->[1] , '' );
-        $_decryptor = Mcrypt::mcrypt_load($ciphers{$_method}->[0] ,'',$ciphers{$_method}->[1], '' );
-        $_key = _EVP_BytesToKey( $ciphers{$_method}->[2], $ciphers{$_method}->[3], '', $_password, 1 );
-        $_iv = makerandom_octet( Length => $ciphers{$_method}->[3] );
-    }
-    else
+    if (!defined($_method))
     {
+        AE::log info => "Encryption method undefinde, using RC4-MD5." ;
         $_encryptor = Mcrypt::mcrypt_load(Mcrypt::ARCFOUR,'',Mcrypt::STREAM , '' );
         $_decryptor = Mcrypt::mcrypt_load(Mcrypt::ARCFOUR ,'',Mcrypt::STREAM, '' );
         $_key = _EVP_BytesToKey(16, 16, '', $_password, 1 );
         $_iv = makerandom_octet( Length => 16 );
+        my $md = Digest::MD5->new();
+        $md->add($_key . $_iv);
+        my $encrypt_rc4_key = $md->digest();
+        Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );      
         }
+        else
+        {
+            if($_method eq 'chacha20-ietf')
+            {
+                $_encryptor = Crypt::NaCl::Sodium->stream();
+                $_key = $_encryptor->chacha20_keygen();
+                $_nonce = $_encryptor->chacha20_ietf_nonce();
+                $_decryptor = Crypt::NaCl::Sodium->stream();
+                $_iv = $_nonce;
+                }
+            elsif ( $_method eq 'rc6') 
+            {
+                $_key = Net::Shadowsocks::_EVP_BytesToKey($_ciphers{$_method}->[2], $_ciphers{$_method}->[3] , '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+                $_encryptor = Crypt::Mode::CBC->new('RC6');
+                $_decryptor = Crypt::Mode::CBC->new('RC6');
+            }
+            elsif(defined($_method) and $_method =~ /cfb$/)
+            {
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_encryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CFB->new($_ciphers{$_method}->[0]);
+            }
+             elsif(defined($_method) and $_method =~ /ctr$/)
+            {
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] ); 
+                $_encryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
+                $_decryptor = Crypt::Mode::CTR->new($_ciphers{$_method}->[0]);
+            }
+            elsif(defined($_method) and $_method !~ /cfb$/ and $_method !~ /ctr$/ )
+            {
+                $_encryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0],'',$_ciphers{$_method}->[1] , '' );
+                $_decryptor = Mcrypt::mcrypt_load($_ciphers{$_method}->[0] ,'',$_ciphers{$_method}->[1], '' );
+                $_key = _EVP_BytesToKey( $_ciphers{$_method}->[2], $_ciphers{$_method}->[3], '', $_password, 1 );
+                $_iv = makerandom_octet( Length => $_ciphers{$_method}->[3] );
+                if ($_method eq 'rc4-md5' )
+                {
+                   my $md = Digest::MD5->new();
+                   $md->add($_key . $_iv);
+                   my $encrypt_rc4_key = $md->digest();
+                   Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
+                }
+                elsif($_method eq 'rc4-sha' )
+                {
+                   my $sha = Digest::SHA->new();
+                   $sha->add($_key . $_iv);
+                   my $encrypt_rc4_key = substr($sha->digest(),0,16);
+                   Mcrypt::mcrypt_init($_encryptor, $encrypt_rc4_key, '' );
+                }
+                elsif($_method ne 'rc6')
+                {
+                   Mcrypt::mcrypt_init($_encryptor, $_key, $_iv );
+                }
+           }
+    }
     return $_encryptor, $_decryptor, $_key, $_iv, $_nonce;
 }
+
+sub _get_algorithm($)
+{
+    my $_method   = shift;
+    return $_ciphers{$_method}->[0];
+    }
+
+sub _get_mode($)
+{
+    my $_method   = shift;
+    return $_ciphers{$_method}->[1];
+    }
+
 1;    # End of Net::Shadowsocks
 
  __END__
 
 =pod
+
 =encoding utf8
+
 =head1 NAME
 
 Net::Shadowsocks
 
 =head1 VERSION
 
-Version 0.7.1
+Version 0.7.2
 
 =head1 SYNOPSIS
 
     Shadowsocks protocol client and server module.
     
-The following shadowsocks encryption methods are supported:  
-
-rc4-md5 aes-128-cbc aes-128-cfb aes-128-ofb aes-256-cbc aes-256-cfb aes-256-ofb
-
-This module also supports rc4-sha and rc6 encryption methods which are not available
-from other shadowsocks implementations.
- 
-Experimental server TCP Fast Open is also supported if available on Mac OS X and Linux. 
-
-Please also note that both the stock libmcrypt and stock Mcrypt module do not export CTR 
-mode, So the aes-128-ctr and aes-256-ctr method will not work out of box. You have to use this 
-  patched version of libmcrypt and Libmcrypt module from https://osdn.net/projects/mcrypt/
-to get it to work.  
-
 =head1 DESCRIPTION
 
 Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭)
 protocol client and server , Shadowsocks is a secure transport protocol based on
 SOCKS Protocol Version 5 (RFC 1928 ). 
 
-Design goal of this module is to provide same level of transport security as TLS 
-1.2, not more,not less. So it does not implement all the encryption methods 
-supported by other Shadowsocks implementations if It is not also a TLS 1.2 cipher. 
-To sum it up:
+1. A total of 28 encryption methods are supported:
 
-1. The following encryption methods are supported:
-
-RC4-MD5 RC4-SHA AES-128-CBC AES-128-CFB AES-128-OFB AES-256-CBC 
-AES-256-CFB AES-256-OFB
-
-experimental support of RC6 is also provided.
+AES-128-CBC AES-128-CFB AES-128-CTR AES-128-OFB 
+AES-192-CBC AES-192-CFB AES-192-CTR AES-192-OFB 
+AES-256-CBC AES-256-CFB AES-256-CTR AES-256-OFB
+Camellia-128-CBC Camellia-128-CFB Camellia-128-CTR Camellia-128-OFB 
+Camellia-192-CBC Camellia-192-CFB Camellia-192-CTR Camellia-192-OFB 
+Camellia-256-CBC Camellia-256-CFB Camellia-256-CTR Camellia-256-OFB
+Chacha20-IETF
+RC4-MD5 RC4-SHA RC6 
 
 2.The following ciphers deprecated by Shadowsocks are not supported: 
 bf-cfb chacha20 salsa20
 
-3.The following ciphers not recommended by Shadowsocks are not  supported: 
-aes-192-cfb aes-192-ctr camellia-128-cfb camellia-192-cfb camellia-256-cfb 
-chacha20-ietf 
-
-4.The following ciphers recommended by Shadowsocks are not supported: 
-aes-128-gcm aes-192-gcm aes-256-gcm chacha20-ietf-poly1305 
+3.The following ciphers recommended by Shadowsocks are not supported yet: 
+aes-128-gcm aes-192-gcm aes-256-gcm 
+chacha20-ietf-poly1305 
 xchacha20-ietf-poly1305 
 
-The following table shows TLS 1.2 cipher suite definitions and  equivalent
-encryptions methods used in this module: 
-
-TLS 1.2 Cipher Suite Definitions              Cipher                Encryption sMethod
-                                                                                                           
-TLS_RSA_WITH_RC4_128_MD5                RC4_128            RC4-MD5
-TLS_RSA_WITH_RC4_128_SHA                 RC4_128            RC4-SHA
-TLS_RSA_WITH_AES_128_CBC_SHA        AES_128_CBC    AES-128-CBC/CFB/OFB
-TLS_RSA_WITH_AES_256_CBC_SHA        AES_256_CBC    AES-256-CBC/CFB/OFB
-TLS_RSA_WITH_AES_128_CBC_SHA256  AES_128_CBC    AES-128-CBC/CFB/OFB
-TLS_RSA_WITH_AES_256_CBC_SHA256  AES_256_CBC    AES-256-CBC/CFB/OFB
-
 Please note TLS 1.2 has removed IDEA and DES cipher suites. and because of 
 CVE-2016-2183,  http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-2183
 , this module has removed all support for DES and 3DES ciphers.
 
-Please also note that both the stock libmcrypt and stock Mcrypt module do not export CTR 
-mode, So the aes-128-ctr and aes-256-ctr method will not work out of box. You have to use this 
-  patched version of libmcrypt and Libmcrypt module from https://osdn.net/projects/mcrypt/
-to get it to work.  
-
 Project website https://osdn.net/projects/ssperl/
 
 =head1 AUTHOR
