@@ -6,6 +6,7 @@ package Net::Shadowsocks::Client;
     use warnings;
     use AnyEvent;
     use AnyEvent::Handle;
+    use AnyEvent::Log;
     use AnyEvent::Socket;
     use Carp;
     use Config;
@@ -13,15 +14,16 @@ package Net::Shadowsocks::Client;
     use Digest::SHA;
     use IO::Socket::Socks qw(:constants $SOCKS_ERROR ESOCKSPROTO);
     use Net::Shadowsocks;
-    use Net::Shadowsocks qw(_EVP_BytesToKey _initialize_cipher);
-    use Socket qw(IPPROTO_TCP MSG_FASTOPEN);
+    use Net::Shadowsocks qw( _get_algorithm _get_mode _EVP_BytesToKey _initialize_cipher);
+    use Socket qw(IPPROTO_TCP MSG_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
 
-    our $VERSION = '0.7.1';
+    our $VERSION = '0.8.0';
+    $AnyEvent::Log::FILTER->level ("info");
     sub new($$$$$$) 
     {
         my $_osname = $Config{osname};
         my $_osvers = $Config{osvers};
-        AE::log info => "Shadowsocks server starting up on $_osname $_osvers";
+        AE::log info => "Shadowsocks local server starting up on $_osname $_osvers";
         my ( $class, %args ) = @_;
         my $self = bless {
             map { ( $_ => $args{$_} ) }
@@ -47,7 +49,8 @@ package Net::Shadowsocks::Client;
                 my $stage = 0;
                 my $mode  = 0;
                 my ( $encryptor, $decryptor, $key, $iv, $nonce ) = Net::Shadowsocks::_initialize_cipher( $self->{method},$self->{password} );
-
+                carp $key;
+                carp $iv;
                 my $clienthandler;
                 my $remotehandler;
                 $clienthandler = AnyEvent::Handle->new
@@ -135,9 +138,9 @@ package Net::Shadowsocks::Client;
                                         connect   => [$self->{server},$self->{server_port}],
                                         on_connect => sub 
                                         {
-                                            my ($rhandle,  $peerhost,$peerport, $retry) = @_;
+                                            my ($chandle,  $peerhost,$peerport, $retry) = @_;
 
-                                            unless ($rhandle) 
+                                            unless ($chandle) 
                                             {
                                                 AE::log info =>"couldn't connect: $!";
                                                 return;
@@ -145,18 +148,23 @@ package Net::Shadowsocks::Client;
                                         },
                                         on_eof => sub 
                                         {
-                                            my $rhandler = shift;
-                                            $rhandler->destroy();
+                                            my $rhandle = shift;
+                                            $rhandle->destroy();
                                             $mode = 0;
                                             AE::log info => "Remote: Done.";
                                         },
                                         on_error => sub 
                                         {
-                                            my ( $rhandler, $fatal, $msg ) = @_;
+                                            my ( $rhandle, $fatal, $msg ) = @_;
                                             AE::log error => $msg;
                                             $mode = 0;
-                                            $rhandler->destroy();
+                                            $rhandle->destroy();
                                         },
+                                        on_prepare => sub
+                                        {
+                                            my $phandle = shift;
+                                            setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                            },
                                         on_read => sub 
                                         {
                                             my $incomingdata = $remotehandler->rbuf();
@@ -165,7 +173,7 @@ package Net::Shadowsocks::Client;
                                                 {
                                                     $server_iv = substr( $incomingdata, 0,length($iv) );
                                                     $incomingdata = substr($incomingdata,length($iv));
-                                                    if ( $self->{method} eq'rc4-md5' )
+                                                    if ( !defined($self->{method}) or $self->{method} eq'rc4-md5' )
                                                     {
                                                         my $md = Digest::MD5->new();
                                                         $md->add($key . $server_iv);
@@ -181,15 +189,27 @@ package Net::Shadowsocks::Client;
                                                         my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                                         Mcrypt::mcrypt_init($decryptor,$decrypt_rc4_key,'' );
                                                         }
-                                                        elsif($self->{method} ne'rc6')
+=for comment
+                                                            elsif($self->{method} =~ /ctr$/)
+                                                            {
+                                                                $decryptor =   Crypt::Nettle::Cipher->new('decrypt', Net::Shadowsocks::_get_algorithm($self->{method}), $key, Net::Shadowsocks::_get_mode($self->{method}) ,$server_iv);                                                                
+                                                                }
+=cut
+                                                                  elsif($self->{method} ne'rc6' and $self->{method} ne 'chacha20-ietf' and $self->{method} != /cfb$/ and $self->{method} != /ctr$/  )
                                                         {
                                                             Mcrypt::mcrypt_init($decryptor,$key,$server_iv );
                                                             }
                                                 }
-                                                if ($self->{method} eq 'rc6')
+                                                if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/)
                                                 {
                                                     $decrypteddata = $decryptor->decrypt($incomingdata,$key,$server_iv);
                                                     }
+
+                                                    elsif($self->{method} eq 'chacha20-ietf')
+                                                    {
+                                                        $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$server_iv,$key);
+                                                        }
+
                                                     else
                                                     {
                                                 $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
@@ -228,44 +248,25 @@ package Net::Shadowsocks::Client;
                             {
                                 my $plaindata = $client_buffer;
                                 my $encrypteddata;
-                                if ( $self->{method} eq 'rc6' ) 
-                                    {
-                                          if ( $addr_to_send ne '' ) 
-                                          {
-                                             $plaindata = $addr_to_send . $client_buffer; 
-                                              $addr_to_send = '';
-                                          }                                   
-                                          $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv);
-                                    }
-                                    else
-                                    {
                                          if ( $addr_to_send ne '' ) 
                                          {
                                              $plaindata = $addr_to_send . $client_buffer;
-                                             if ( $self->{method} eq 'rc4-md5' )
-                                             {
-                                                 my $md = Digest::MD5->new();
-                                                 $md->add($key . $iv);
-                                                 #$md->add($iv);
-                                                 my $encrypt_rc4_key = $md->digest();
-                                                 Mcrypt::mcrypt_init($encryptor, $encrypt_rc4_key, '' );
-                                            }
-                                            elsif($self->{method} eq 'rc4-sha' )
-                                            {
-                                                my $sha = Digest::SHA->new();
-                                                 $sha->add($key . $iv);
-                                                 #$md->add($iv);
-                                                 my $encrypt_rc4_key = substr($sha->digest(),0,16);
-                                                 Mcrypt::mcrypt_init($encryptor, $encrypt_rc4_key, '' );
-                                                }
-                                            else 
-                                            {
-                                                 Mcrypt::mcrypt_init($encryptor, $key, $iv );
-                                            }
                                             $addr_to_send = '';
                                      }
+                                      if ( $self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/) 
+                                    {
+                                                                    
+                                          $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv);
+                                    }
+                                     elsif($self->{method} eq 'chacha20-ietf')
+                                                    {
+                                                        $encrypteddata = $encryptor->chacha20_ietf_xor($plaindata,$nonce,$key);
+                                                        }
+                                    else
+                                    {
                                      $encrypteddata = Mcrypt::mcrypt_encrypt($encryptor,$plaindata);
-                                }
+                                 }
+                                
                                 my $datatosend;
                                 if ( $mode == 0  ) 
                                 {
@@ -288,7 +289,12 @@ package Net::Shadowsocks::Client;
                         $clienthandler->{rbuf} = '';
                     },
                 );
-            }
+            },
+            sub
+            {
+                my ($fh) = shift;
+                setsockopt($fh, SOL_SOCKET, SO_REUSEADDR, 1);
+                }
         );
         my $cv = AE::cv;
         $cv->recv();
@@ -299,30 +305,47 @@ package Net::Shadowsocks::Client;
   __END__
     
 =pod
+
 =encoding utf8
+
 =head1 NAME
 
 Net::Shadowsocks::Client
 
 =head1 VERSION
 
-Version 0.7.1
+Version 0.8.0
 
 =head1 SYNOPSIS
 
-    Shadowsocks protocol client module.
+    Shadowsocks protocol server module.
+    
+=head1 DESCRIPTION
 
-The following shadowsocks encryption methods are supported:  
+1. A total of 28 encryption methods are supported:
 
-rc4-md5 aes-128-cbc aes-128-cfb aes-128-ofb aes-256-cbc aes-256-cfb aes-256-ofb
+AES-128-CBC AES-128-CFB AES-128-CTR AES-128-OFB 
+AES-192-CBC AES-192-CFB AES-192-CTR AES-192-OFB 
+AES-256-CBC AES-256-CFB AES-256-CTR AES-256-OFB
+Camellia-128-CBC Camellia-128-CFB Camellia-128-CTR Camellia-128-OFB 
+Camellia-192-CBC Camellia-192-CFB Camellia-192-CTR Camellia-192-OFB 
+Camellia-256-CBC Camellia-256-CFB Camellia-256-CTR Camellia-256-OFB
+Chacha20-IETF
+RC4-MD5 RC4-SHA RC6 
 
-This module also supports rc4-sha and rc6 encryption methods which are not available
-from other shadowsocks implementations.
+2.The following ciphers deprecated by Shadowsocks are not supported: 
+bf-cfb chacha20 salsa20
 
-Please also note that both the stock libmcrypt and stock Mcrypt module do not export CTR 
-mode, So the aes-128-ctr and aes-256-ctr method will not work out of box. You have to use this 
-  patched version of libmcrypt and Libmcrypt module from https://osdn.net/projects/mcrypt/
-to get it to work.  
+3.The following ciphers recommended by Shadowsocks are not supported yet: 
+aes-128-gcm aes-192-gcm aes-256-gcm 
+chacha20-ietf-poly1305 
+xchacha20-ietf-poly1305 
+
+Please note TLS 1.2 has removed IDEA and DES cipher suites. and because of 
+CVE-2016-2183,  http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-2183
+, this module has removed all support for DES and 3DES ciphers.
+
+Project website https://osdn.net/projects/ssperl/
 
 =head1 METHODS
 
