@@ -1,24 +1,30 @@
  package Net::Shadowsocks::Server;
 
     # ABSTRACT: Shadowsocks protocol server module.
+    use 5.006;
+    use strict;
+    use warnings;
     use AnyEvent;
     use AnyEvent::Handle;
+    use AnyEvent::Log;
     use AnyEvent::Socket;
     use Carp;
     use Config;
     use Digest::MD5;
     use Digest::SHA;
     use Net::Shadowsocks;
-    use Net::Shadowsocks qw(_EVP_BytesToKey _initialize_cipher);
-    use Socket qw(IPPROTO_TCP TCP_FASTOPEN);
+    use Net::Shadowsocks qw(_get_algorithm _get_mode _EVP_BytesToKey _initialize_cipher);
+    use Socket qw(IPPROTO_TCP TCP_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
 
-    our $VERSION = '0.7.1';
+    our $VERSION = '0.8.0';
+    
+    $AnyEvent::Log::FILTER->level ("info");
     
     sub new($$$$$$)
      {
         my $_osname = $Config{osname};
         my $_osvers = $Config{osvers};
-        AE::log info => "Shadowsocks server starting up on $_osname $_osvers";
+        AE::log info => "Shadowsocks remote server starting up on $_osname $_osvers";
         my ( $class, %args ) = @_;
         my $self = bless {
             map { ( $_ => $args{$_} ) }
@@ -37,9 +43,7 @@
             sub {
                 my ( $client_socket, $client_host, $client_port ) = @_;
 
-                AE::log info =>
-                  "Got new client connection: $client_host:$client_port";
-                my $sent_iv = 0;
+                AE::log info => "Got new client connection: $client_host:$client_port";
                 my $client_iv;
                 my $mode = 0;
 
@@ -81,7 +85,7 @@
                                     $client_iv = substr( $client_buffer, 0, length($iv) );
                                     $incomingdata = substr( $client_buffer, length($iv) );
 
-                                    if ( $self->{method} eq 'rc4-md5' ) 
+                                    if ( !defined($self->{method}) or $self->{method} eq 'rc4-md5' ) 
                                     {
                                         my $md = Digest::MD5->new();
                                         $md->add($key .$client_iv );
@@ -97,13 +101,25 @@
                                         my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                         Mcrypt::mcrypt_init($decryptor, $decrypt_rc4_key, '' );
                                         }
-                                    if ($self->{method} eq 'rc6')
+                                        elsif($self->{method} ne 'rc6' and $self->{method} ne 'chacha20-ietf' and  $self->{method} !~ /cfb$/ and $self-> {method} !~ /ctr$/ )
+                                        {
+                                            Mcrypt::mcrypt_init($decryptor, $key, $client_iv );
+                                            }
+                                            else
+                                            {
+                                                }                                    
+                                    if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/)
                                     {
                                     $decrypteddata = $decryptor->decrypt($incomingdata,$key,$client_iv);
                                         }
+                                           elsif($self->{method} eq 'chacha20-ietf')
+                                       {
+                                           carp "chacha20 ietf mode";
+                                           #$decrypteddata = $decryptor->process($incomingdata);
+                                           $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$client_iv,$key);
+                                       }
                                         else 
                                     {
-                                        Mcrypt::mcrypt_init($decryptor, $key, $client_iv );
                                     $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
                                      }                                
                                 my $addrtype = ord( substr( $decrypteddata, 0, 1 ) );
@@ -162,9 +178,12 @@
                                         last;
                                     }
                                 }
+                                 carp $dest_addr;
+                                 carp $dest_port;
 
                                 if ( $dest_addr eq '' ) 
                                 {
+                                    carp "dest_addr empty";
                                     last;
                                 }
                                 $remotehandler = AnyEvent::Handle->new
@@ -197,41 +216,32 @@
                                         $mode = 0;
                                         $rhandler->destroy();
                                     },
+                                     on_prepare => sub
+                                        {
+                                            my $phandle = shift;
+                                            setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                            },
                                     on_read => sub 
                                     {
                                         my $remote_buffer = $remotehandler->rbuf();
                                         my $plaindata = $remote_buffer;
                                         my $encrypteddata;
-                                        if ( $sent_iv == 0 ) 
-                                        {
 
-                                            if ( $self->{method} eq 'rc4-md5' )
-                                            {
-                                                my $md = Digest::MD5->new();
-                                                $md->add($key . $iv);
-                                                #$md->add($iv);
-                                                my $encrypt_rc4_key = $md->digest();
-                                                Mcrypt::mcrypt_init($encryptor,$encrypt_rc4_key, '' );
-                                            }
-                                            elsif($self->{method} eq 'rc4-sha' )
-                                            {
-                                                my $sha = Digest::SHA->new();
-                                                $sha->add($key . $iv);
-                                                #$md->add($iv);
-                                                my $encrypt_rc4_key = substr($sha->digest(),0,16);
-                                                Mcrypt::mcrypt_init($encryptor,$encrypt_rc4_key, '' );
-                                                }
-                                            if ($self->{method} eq 'rc6')
+                                            if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/)
                                             {
                                                 $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv); 
                                                 }
+
+                                            elsif($self->{method} eq 'chacha20-ietf')
+                                                    {
+                                                        #$encrypteddata = $encryptor->process($plaindata);
+                                                        $encrypteddata = $encryptor->chacha20_ietf_xor($plaindata,$iv,$key);
+                                                        }
                                                 else 
                                             {
-                                                Mcrypt::mcrypt_init($encryptor, $key, $iv );
                                                 $encrypteddata = Mcrypt::mcrypt_encrypt($encryptor, $plaindata );
                                             }
-                                            $sent_iv = 1;
-                                        }
+                                        
                                         my $datatosend;
                                         if ( $mode == 0 ) 
                                         {
@@ -249,10 +259,16 @@
                             }
                             else 
                             {
-                                if ( $self->{method} eq 'rc6' ) 
+                                if ( $self->{method} eq 'rc6'  or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/) 
                                 {
                                     $decrypteddata = $decryptor->decrypt($incomingdata,$key,$client_iv);
                                 }
+
+                                 elsif($self->{method} eq 'chacha20-ietf')
+                                       {
+                                           #$decrypteddata = $decryptor->process($incomingdata);
+                                            $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$client_iv,$key);
+                                       }
                                 else 
                                 {
                                     $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
@@ -267,6 +283,7 @@
             sub 
             {
                 my $fh = shift;
+                setsockopt($fh, SOL_SOCKET, SO_REUSEADDR, 1);
                 if ( $_osname eq 'linux' ) 
                 {
                     my $_tfo = do 
@@ -299,6 +316,7 @@
   __END__ 
     
 =pod
+
 =encoding utf8
     
 =head1 NAME
@@ -307,25 +325,38 @@ Net::Shadowsocks::Server
 
 =head1 VERSION
 
-Version 0.7.1
+Version 0.8.0
 
 =head1 SYNOPSIS
 
     Shadowsocks protocol server module.
     
-The following shadowsocks encryption methods are supported:  
+=head1 DESCRIPTION
 
-rc4-md5 aes-128-cbc aes-128-cfb aes-128-ofb aes-256-cbc aes-256-cfb aes-256-ofb
+1. A total of 28 encryption methods are supported:
 
-This module also supports rc4-sha and rc6 encryption methods which are not available
-from other shadowsocks implementations.
+AES-128-CBC AES-128-CFB AES-128-CTR AES-128-OFB 
+AES-192-CBC AES-192-CFB AES-192-CTR AES-192-OFB 
+AES-256-CBC AES-256-CFB AES-256-CTR AES-256-OFB
+Camellia-128-CBC Camellia-128-CFB Camellia-128-CTR Camellia-128-OFB 
+Camellia-192-CBC Camellia-192-CFB Camellia-192-CTR Camellia-192-OFB 
+Camellia-256-CBC Camellia-256-CFB Camellia-256-CTR Camellia-256-OFB
+Chacha20-IETF
+RC4-MD5 RC4-SHA RC6 
 
-Experimental server TCP Fast Open is also supported if available on Mac OS X and Linux. 
+2.The following ciphers deprecated by Shadowsocks are not supported: 
+bf-cfb chacha20 salsa20
 
-Please also note that both the stock libmcrypt and stock Mcrypt module do not export CTR 
-mode, So the aes-128-ctr and aes-256-ctr method will not work out of box. You have to use this 
-  patched version of libmcrypt and Libmcrypt module from https://osdn.net/projects/mcrypt/
-to get it to work.  
+3.The following ciphers recommended by Shadowsocks are not supported yet: 
+aes-128-gcm aes-192-gcm aes-256-gcm 
+chacha20-ietf-poly1305 
+xchacha20-ietf-poly1305 
+
+Please note TLS 1.2 has removed IDEA and DES cipher suites. and because of 
+CVE-2016-2183,  http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-2183
+, this module has removed all support for DES and 3DES ciphers.
+
+Project website https://osdn.net/projects/ssperl/
 
 =head1 METHODS
 
